using System;
using System.Runtime.Serialization;
using System.Collections;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.OrgLayout
{
	/// <summary>
	/// Each employee in the organization will be represented in the diagram as a Symbol node
	/// </summary>
	[Serializable]
	public class EmployeeSymbol : Symbol, ISerializable
	{
		private RoundRect rect = null;
		private Label nameLbl = null;
		private Label titleLbl = null;
		private String strManagerID = String.Empty;
		private int nRecSubEmployeeCount = 0;

		public EmployeeSymbol()
		{
			this.rect = new RoundRect(0, 0, 60, 25);
			this.LineStyle.LineWidth = 1;
			this.rect.FillStyle.Type = Syncfusion.Windows.Forms.Diagram.FillStyle.FillType.LinearGradient;
			this.rect.FillStyle.GradientStartColor =  System.Drawing.Color.Orange;
			this.rect.FillStyle.Color = System.Drawing.Color.OrangeRed;
			this.AppendChild(this.rect);
			this.nameLbl = this.AddLabel("", BoxPosition.Center);
			this.nameLbl.OffsetY = -5;
			this.nameLbl.FontStyle.Name = "Verdana";
			this.nameLbl.FontStyle.Size = 8;
			this.nameLbl.FontStyle.Style = System.Drawing.FontStyle.Regular;
			this.nameLbl.BackgroundStyle.Color = System.Drawing.Color.Transparent;
			this.nameLbl.ReadOnly = true;
			this.nameLbl.FitBlackBox = true;
			this.nameLbl.WrapText = false;
			this.nameLbl.NoClip = false;
			this.titleLbl = this.AddLabel("", BoxPosition.Center);
			this.titleLbl.FontStyle.Name = "Verdana";
			this.titleLbl.FontStyle.Size = 7;
			this.titleLbl.FontStyle.Style = System.Drawing.FontStyle.Italic;
			this.titleLbl.OffsetY = 7;
			this.titleLbl.BackgroundStyle.Color = System.Drawing.Color.Transparent;
			this.titleLbl.FitBlackBox = true;
			this.titleLbl.WrapText = false;
			this.titleLbl.NoClip = false;
			this.titleLbl.ReadOnly = true;
		}

		public EmployeeSymbol(EmployeeSymbol src) : base(src)
		{
		}

		// Serialization constructor
		protected EmployeeSymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			this.strManagerID = info.GetString("ManagerID");
			this.nRecSubEmployeeCount = info.GetInt32("RecSubEmployeeCount");
		}

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);

			info.AddValue("ManagerID", this.strManagerID);
			info.AddValue("RecSubEmployeeCount", this.nRecSubEmployeeCount);
		}

		public string EmployeeName
		{
			get
			{
				return this.nameLbl.Text;
			}
			set
			{
				this.nameLbl.Text = value;
				this.nameLbl.Size = this.nameLbl.SizeToText(this.Size);
			}
		}

		public string EmployeeID
		{
			get
			{
				return this.titleLbl.Text;
			}
			set
			{
				this.titleLbl.Text = value;
				this.titleLbl.Size = this.titleLbl.SizeToText(this.Size);
			}
		}

		public string ManagerID
		{
			get { return this.strManagerID; }
			set
			{
				if(this.strManagerID != value)
					this.strManagerID = value;
			}
		}

		public override bool AcceptConnection(Port sourcePort, Port targetPort)
		{
			bool accept = false;

			// First, determine which port belongs to the foreign port container.
			Port foreignPort = null;
			if(this.Ports.Contains(sourcePort) || this.CenterPort == sourcePort)
			{
				foreignPort = targetPort;
			}
			else if (this.Ports.Contains(targetPort) || this.CenterPort == targetPort)
			{
				foreignPort = sourcePort;
			}

			if (foreignPort == null)
			{
				return false;  // This should never happen
			}

			// Now get the foreign port container and check to see if it is
			// a Link object. If so, then check to see if the foreign port
			// is the link's tail port or head port. If it's the tail port,
			// then always allow the connection, because a member symbol can
			// have any number of edges leaving it. If it's the head port,
			// then only allow the connection if the member symbol has no
			// other edges entering it yet.

			IPortContainer foreignContainer = foreignPort.Container;
			if (foreignContainer != null)
			{
				Link link = foreignContainer as Link;
				if (link != null)
				{
					if (foreignPort == link.TailPort)
					{
						accept = true;
					}
					else
					{
						IGraphNode curNode = this as IGraphNode;

						if (curNode != null)
						{
							ICollection edgesEntering = curNode.EdgesEntering;
							if (edgesEntering.Count == 0)
							{
								accept = true;
							}
						}
					}
				}
			}

			return accept;
		}

		public int Rank
		{
			get
			{
				int rank = 1;

				IGraphNode thisNode = this as IGraphNode;
				IGraphNode curNode = thisNode;
				bool cyclicalPathFound = false;

				while (curNode != null && !cyclicalPathFound)
				{
					ICollection edgesEntering = curNode.EdgesEntering;
					if (edgesEntering.Count > 0)
					{
						IEnumerator enumEdgesEntering = edgesEntering.GetEnumerator();
						if (enumEdgesEntering != null)
						{
							enumEdgesEntering.MoveNext();
							IGraphEdge incomingEdge = enumEdgesEntering.Current as IGraphEdge;
							if (incomingEdge != null)
							{
								curNode = incomingEdge.FromNode;

								if (curNode == thisNode)
								{
									cyclicalPathFound = true;
								}
								else
								{
									rank++;
								}
							}
						}
					}
					else
					{
						curNode = null;
					}
				}

				return rank;
			}
		}

		public EmployeeSymbol Manager
		{
			get
			{
				EmployeeSymbol manager = null;
				IGraphNode thisNode = this as IGraphNode;
				IGraphNode fromNode;

				if (thisNode != null)
				{
					ICollection edgesEntering = thisNode.EdgesEntering;
					if (edgesEntering.Count > 0)
					{
						IEnumerator enumEdgesEntering = edgesEntering.GetEnumerator();
						if (enumEdgesEntering != null)
						{
							enumEdgesEntering.MoveNext();
							IGraphEdge incomingEdge = enumEdgesEntering.Current as IGraphEdge;
							if (incomingEdge != null)
							{
								fromNode = incomingEdge.FromNode;
								if (fromNode != null)
								{
									manager = fromNode as EmployeeSymbol;
								}
							}
						}
					}
				}

				return manager;
			}
		}

		public ICollection Employees
		{
			get
			{
				ArrayList employees = new ArrayList();
				IGraphNode thisNode = this as IGraphNode;
				if(thisNode != null)
				{
					ICollection edgesLeaving = thisNode.EdgesLeaving;
					if (edgesLeaving != null)
					{
						foreach (IGraphEdge curEdge in edgesLeaving)
						{
							EmployeeSymbol curEmployee = curEdge.ToNode as EmployeeSymbol;
							if (curEmployee != null)
							{
								employees.Add(curEmployee);
							}
						}
					}
				}

				return employees;
			}
		}
		
		public int RecSubEmployeeCount
		{
			get { return this.nRecSubEmployeeCount; }
			set { this.nRecSubEmployeeCount = value; }
		}		
	}
}
