using System;
using System.Collections;
using System.ComponentModel;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.OrgLayout
{
	/// <summary>
	/// The OrgChartManager class is an implementor of the Essential Diagram LayoutManager functionality.
	/// This is a very simple layout management algorithm that calculates the sub-employees associated with 
	/// each employee and allocates space accordingly. More efficient management strategies may be used for 
	/// better diagram layout.
	/// </summary>
	public class OrgChartManager : LayoutManager
	{
		public OrgChartManager()
		{
		}

		public OrgChartManager(Model mdl) : base(mdl)
		{
		}

		public override bool UpdateLayout(object contextInfo)
		{
			bool changesMade = true;

			if(this.Model != null)
			{
				ArrayList topRanking = new ArrayList();

				foreach (INode curNode in this.Model.Nodes)
				{
					EmployeeSymbol memberSym = curNode as EmployeeSymbol;
					if(memberSym != null)
					{
						if(memberSym.Rank == 1)
						{
							topRanking.Add(memberSym);
						}
					}
				}

				int numTopRanking = topRanking.Count;

				if(numTopRanking > 0)
				{
					float range = this.Model.Width/((float)numTopRanking);
					float curLeft = this.Model.Bounds.Left;
					float curRight = curLeft;
					foreach(EmployeeSymbol curMember in topRanking)
					{
						// Assuming a 4 pixel bufer between adjacent symbols, the space allocated for each of the 
						// top ranking symbols is calculated based on a recursive count of the subemployees associated 
						// with that top level employee.
						int numsubemployees = curMember.RecSubEmployeeCount;					
						if(numsubemployees <= 0)
							numsubemployees = 1;
						curRight += ((curMember.Width+4) * numsubemployees);
						this.PositionMember(curMember, curLeft, curRight);
						curLeft = curRight;						
					}
				}
			}

			if(changesMade)
			{
				this.OnLayoutUpdated(new System.EventArgs());
			}

			return changesMade;
		}

		private void PositionMember(EmployeeSymbol memberSym, float left, float right)
		{
			int rank = memberSym.Rank;
			if(rank == 1)
				memberSym.Y = verticalStartOffset;
			else
				memberSym.Y = verticalStartOffset + ((rank-1) * this.verticalSpacing);
			float symWidth = memberSym.Width;
			float rangeCenter = (left + right) / 2.0f;
			memberSym.X = rangeCenter - (symWidth / 2.0f);

			ICollection employees = memberSym.Employees;
			int numEmployees = memberSym.RecSubEmployeeCount;					
			if(numEmployees > 0)
			{
				float curLeft = left;
				float curRight = curLeft;				
				foreach(EmployeeSymbol employee in employees)
				{
					int numsubemployees = employee.RecSubEmployeeCount;
					if(numsubemployees <= 0)
						numsubemployees = 1;
					curRight += ((employee.Width+4) * numsubemployees);
					this.PositionMember(employee, curLeft, curRight);
					curLeft = curRight;					
				}
			}
		}
		
		public float HorizontalSpacing
		{
			get { return this.horizontalSpacing; }
			set	{ this.horizontalSpacing = value; }
		}

		public float VerticalSpacing
		{
			get { return this.verticalSpacing; }
			set	{ this.verticalSpacing = value; }
		}

		private float verticalStartOffset = 50.0f;
		private float horizontalSpacing = 100.0f;
		private float verticalSpacing = 140.0f;
	}
}
